// This file may be redistributed and modified only under the terms of
// the GNU General Public License (See COPYING for details).
// Copyright (C) 2003 - 2006 Simon Goodall

package WFUT;

import java.io.*;
import java.util.*;
import javax.xml.parsers.*;

import org.xml.sax.*;
import org.xml.sax.helpers.DefaultHandler;

public class ChannelParser {
  public static class ProtoHandler extends DefaultHandler {
    private static final String TAG_channel = "channel";

    private static final String TAG_name = "name";

    private static final String TAG_description = "description";

    private static final String TAG_url = "url";

    private static final String TAG_email = "email";

    private static final String TAG_logo = "logo";

    // private Locator locator;
    // public void setDocumentLocator(Locator locator) { this.locator = locator;
    // }

    public void startElement(String nameSpaceURI, String localName, String rawName, Attributes atts) throws SAXException {

      if (rawName.equals(TAG_channel)) {
        channel = new Channel();
        channels.add(channel);
        for (int i = 0; i < atts.getLength(); ++i) {
          final String name = atts.getQName(i);
          final String value = atts.getValue(i);

          if (name.equals(TAG_name)) {
            channel.setName(value);
          }
        }
      } else if (rawName.equals(TAG_description)) {
        mode = MODE_DESC;
      } else if (rawName.equals(TAG_url)) {
        mode = MODE_URL;
      } else if (rawName.equals(TAG_email)) {
        mode = MODE_EMAIL;
      } else if (rawName.equals(TAG_logo)) {
        mode = MODE_LOGO;
      }
    }

    public void endElement(String uri, String name, String qName) {
      mode = 0;
    }

    public void characters(char ch[], int start, int length) {
      String chrs = new String(ch);
      chrs = chrs.substring(start, start + length);
      if (mode == MODE_DESC) {
        channel.setDescription(chrs);
      } else if (mode == MODE_URL) {
        channel.setURL(chrs);
      } else if (mode == MODE_EMAIL) {
        channel.setEmail(chrs);
      } else if (mode == MODE_LOGO) {
        channel.setLogo(chrs);
      }
    }

    int mode = 0;

    static final int MODE_DESC = 1;

    static final int MODE_URL = 2;

    static final int MODE_EMAIL = 3;

    static final int MODE_LOGO = 4;

    private Channel channel = null;

    public List getChannels() {
      return channels;
    }

    private List channels = new ArrayList();
  }

  public ChannelParser() throws SAXException, ParserConfigurationException {
    SAXParserFactory factory = SAXParserFactory.newInstance();
    factory.setValidating(false);
    factory.setNamespaceAware(false);
    SAXParser parser = factory.newSAXParser();
    // parser.setFeature("http://xml.org/sax/features/validation");
    reader = parser.getXMLReader();
    reader.setFeature("http://xml.org/sax/features/validation", false);
    reader.setFeature("http://xml.org/sax/features/namespaces", false);
    // reader.setFeature("http://xml.org/sax/features/namespaces", false);
    proto = new ProtoHandler();
    reader.setContentHandler(proto);
  }

  public void parse(InputStream is) throws Exception {
    try {
      reader.parse(new InputSource(is));
    } catch (SAXException e) {
      System.out.println(e);
    }
  }

  public void parse(final String xmlString) throws Exception {
    reader.parse(new InputSource(new StringReader(xmlString)));
  }

  public List getChannels() {
    return proto.getChannels();
  }

  private XMLReader reader = null;

  private ProtoHandler proto = null;
}
